@@ -24,23 +24,8 @@ pub mod ts;
 /// since the `struct` is not part of the public interface of the NEAR contract.
 ///
 /// An overview of the `near_bindgen` attribute macro can be found in
-/// https://www.near-sdk.io/contract-structure/near-bindgen.
+/// <https://www.near-sdk.io/contract-structure/near-bindgen>.
 pub trait NearImpl {
-    /// Returns whether the given `self` implementation is marked as `near_bindgen`.
-    /// This should be an indication to further process this `impl` item.
-    ///
-    /// ### Examples
-    ///
-    /// The following `impl` item is marked as `near_bindgen`.
-    ///
-    /// ```compile_fail
-    /// #[near_bindgen]
-    /// impl Counter {
-    ///     // methods...
-    /// }
-    /// ```
-    fn is_bindgen(&self) -> bool;
-
     /// Returns the trait name this `impl` implements, if any.
     fn get_trait_name(&self) -> Option<String>;
 
@@ -51,7 +36,7 @@ pub trait NearImpl {
     /// Returns whether the given `self` implementation has any exported method.
     ///
     /// For more info on public method in the NEAR SDK,
-    /// see https://www.near-sdk.io/contract-interface/public-methods.
+    /// see <https://www.near-sdk.io/contract-interface/public-methods>.
     fn exported_methods(&self) -> Vec<&ImplItemMethod>;
 
     /// Returns the exported methods if `self` `is_bindgen`.
@@ -65,10 +50,6 @@ pub trait NearImpl {
 }
 
 impl NearImpl for ItemImpl {
-    fn is_bindgen(&self) -> bool {
-        has_attr(&self.attrs, "near_bindgen")
-    }
-
     fn get_trait_name(&self) -> Option<String> {
         if let Some((_excl, trait_path, _for)) = &self.trait_ {
             let trait_name = join_path(trait_path);
@@ -191,6 +172,30 @@ impl NearMethod for ImplItemMethod {
     }
 }
 
+///
+pub trait NearBindgen {
+    /// Returns whether the given `self` implementation is marked as `near_bindgen`.
+    /// This should be an indication to further process this `impl` item.
+    ///
+    /// ### Examples
+    ///
+    /// The following `impl` item is marked as `near_bindgen`.
+    ///
+    /// ```compile_fail
+    /// #[near_bindgen]
+    /// impl Counter {
+    ///     // methods...
+    /// }
+    /// ```
+    fn is_bindgen(&self) -> bool;
+}
+
+impl<I: NearAttributable> NearBindgen for I {
+    fn is_bindgen(&self) -> bool {
+        has_attr(&self.attrs(), "near_bindgen")
+    }
+}
+
 /// Defines methods to deal with serde's declarations in `struct`s or `enum`s.
 pub trait NearSerde {
     /// Returns whether the given `self` item derives `serde::Serialize`.
@@ -223,6 +228,12 @@ pub trait NearAttributable {
     fn attrs(&self) -> &Vec<Attribute>;
 }
 
+impl NearAttributable for ItemImpl {
+    fn attrs(&self) -> &Vec<Attribute> {
+        &self.attrs
+    }
+}
+
 impl NearAttributable for ItemStruct {
     fn attrs(&self) -> &Vec<Attribute> {
         &self.attrs
@@ -317,7 +328,7 @@ pub fn get_docs(attrs: &Vec<Attribute>) -> Vec<String> {
 
 /// Writes Rust `doc` comments to `file`.
 /// Each line of `doc` is prefixed with `prefix`.
-/// See https://doc.rust-lang.org/rustdoc/the-doc-attribute.html.
+/// See <https://doc.rust-lang.org/rustdoc/the-doc-attribute.html>.
 pub fn write_docs<W: Write, F: Fn(String) -> String>(
     file: &mut W,
     attrs: &Vec<Attribute>,
